//gcc -lnsl -lresolv -lcurses breakout.c -o breakout && ./breakout 127.0.0.1 1024

#include "TCP_Client.c"
#include <time.h>
#include "curses.h"
#include <error.h>


#define SCREEN_W 25
#define SCREEN_H 11
#define PAD_LEN 5
#define TAIL_LEN 2
#define FIELD_NUM 2

unsigned char** block_array;
int pad_pos;
int ballx, bally, ballvx, ballvy;
unsigned char* tail_buf;


void buildField()
{	
	int i,j;
	switch (FIELD_NUM)
	{
		case 0:
		{
			unsigned char in[] = {255,255,128,128,64,64};
			for (i=0;i<6;i++)
			{
				for (j=2;j<SCREEN_W-3;j++)
					block_array[i+1][j] = in[i];
			}
			break;
		}
		case 1:
		{
			uint64_t in[] = {
			0b0001010101010000001111111111000000010101010100,
			0b0001101010010000001110101011000000011010100100,
			0b0001101110010000001110011011000000011011100100,
			0b0001101110010000001110011011000000011011100100,
			0b0001101010010000001110101011000000011010100100,
			0b0001010101010000001111111111000000010101010100
			};
			
			for (i=0;i<6;i++)
			{
				uint64_t row = in[i];
				for (j=1;j<SCREEN_W-2;j++)
				{
					unsigned char val = (unsigned char)(row & 0b11);
					//printf("%i",val);
					val = val*64;
					block_array[i+1][j] = val;
					row = row >> 2;
				}
				//printf("\n");
			}
			break;
		}
		case 2:
		{
			uint64_t in[] = {
			0b1111111111111111111111111111111111111111111111,
			0b1111111111111111110000000000111111111111111111,
			0b1010101010101000000000000000000000101010101010,
			0b1010101000000000000000000000000000000010101010,
			0b0101000000000000000000000000000000000000000101,
			0b0100000000000000000000000000000000000000000001
			};
			
			for (i=0;i<6;i++)
			{
				uint64_t row = in[i];
				for (j=1;j<SCREEN_W-2;j++)
				{
					unsigned char val = (unsigned char)(row & 0b11);
					//printf("%i",val);
					val = val*64;
					block_array[i+1][j] = val;
					row = row >> 2;
				}
				//printf("\n");
			}
			break;
		}
		default:
		{
			for (i=1;i<SCREEN_H-5;i++)
			{
				for (j=2;j<SCREEN_W-3;j++)
					block_array[i][j] = (rand() % 3) * 64;
			}
			break;
		}
	}
}

void getKey()
{
	int ch=getch();
	while(ch!=ERR)
	{
		int last_pos=pad_pos;
		switch (ch)
		{
			case KEY_LEFT:
			case 'a':
			{
				if (last_pos > 0)
					pad_pos--;
				break;
			}
			case KEY_RIGHT:
			case 'd':
			{
				if (last_pos < SCREEN_W-PAD_LEN-1)
					pad_pos++;
				break;
			}
			case 27:	//ESC
			{
				endwin();
				exit(1);
				break;
			}
			default:
				break;
		}
		ch=getch();
	}
}

void moveBall()
{
	int nextx = ballx + ballvx;
	int nexty = bally + ballvy;
	
	if (nextx < 0)
	{
		ballvx = -ballvx;
		nextx = ballx+1; //for pad
	}
	if (nextx > SCREEN_W)
	{
		ballvx = -ballvx;
		nextx = ballx-1; //for pad
	}
	if (nexty < 0)
	{
		ballvy = -ballvy;
		nexty = bally;
	}
	
	if ((ballvx > 0) && (ballx < SCREEN_W))
	{
		if (block_array[bally][ballx+1])
		{
			block_array[bally][ballx+1] = 0;
			ballvx = -ballvx;
		}
	}
	
	if ((ballvx < 0) && (ballx > 0))
	{
		if (block_array[bally][ballx-1])
		{
			block_array[bally][ballx-1] = 0;
			ballvx = -ballvx;
		}
	}
	
	if ((ballvy > 0) && (bally < SCREEN_H))
	{
		if (block_array[bally+1][ballx])
		{
			block_array[bally+1][ballx] = 0;
			ballvy = -ballvy;
		}
	}
	
	if ((ballvy < 0) && (bally > 0))
	{
		if (block_array[bally-1][ballx])
		{
			block_array[bally-1][ballx] = 0;
			ballvy = -ballvy;
		}
	}
		
	if (nexty == SCREEN_H-1)
	{
		//pad?
		if ((nextx >= pad_pos) && (nextx <= (pad_pos+PAD_LEN-1)))
		{
			int pad_mid = pad_pos + (PAD_LEN-1)/2;
			ballvx += nextx - pad_mid;
			if (ballvx > 1)
				ballvx = 1;
			if (ballvx < -1)
				ballvx = -1;
			ballvy = -ballvy;			
		}
	}
	
	ballx += ballvx;
	bally += ballvy;
}

void writeField()
{
	resetScreen();
	int x,y;
	for (y=0;y<SCREEN_H;y++)
	{
		for (x=0;x<SCREEN_W;x++)
		{
			putPixel(x,y,block_array[y][x]);
		}
	}
	if (TAIL_LEN == 0)
		putPixel(ballx,bally,255);
	else
	{
		for (x=TAIL_LEN;x>=0;x--)
		{
			writeToArray(255+(64-255)*x/TAIL_LEN,tail_buf[x]);
		}
	}
	for(x=0; x<PAD_LEN; x++)
		putPixel(pad_pos+x,SCREEN_H-1,255);
}

void initGame()
{
	resetScreen();
	pad_pos = (SCREEN_W-PAD_LEN)/2;
	ballx = pad_pos + PAD_LEN/2;
	bally = SCREEN_H - 4;
	ballvx = 0;//rand() % 3 - 1;
	ballvy = 1;
	buildField();
	if (TAIL_LEN > 0)
	{
		int i;
		for (i=0; i<TAIL_LEN;i++)
			tail_buf[i] = getPos(ballx,bally);
	}
	
	/*putNumber(3,10,1,255);
	display();
	sleep(1);
	putNumber(2,10,1,255);
	display();
	sleep(1);
	putNumber(1,10,1,255);
	display();
	sleep(1);
	resetScreen();
	putChar('G',7,4,255);
	putChar('O',11,4,255);
	putChar('!',15,4,255);
	display();
	usleep(500000);*/
	while(getch()!=ERR){}
}

int main(int argc, char *argv[])
{
	if(argc!=3)
	{
		printf("Usage: ./breakout <Servername> <Serverport>\n");
		exit(1);
	}
	srand(time(NULL));
	init(argv[1],atoi(argv[2]));
	initscr();
	noecho();
	cbreak();
	keypad(stdscr, TRUE);
	nodelay(stdscr, TRUE);
	
	block_array = malloc(SCREEN_H*sizeof(char*));
	int i;
	for (i=0;i<SCREEN_H;i++)
		block_array[i] = calloc(SCREEN_W,sizeof(unsigned char));
	
	if (TAIL_LEN == 0)
		tail_buf = NULL;
	else
		tail_buf = calloc(TAIL_LEN+1,sizeof(unsigned char));
	
	initGame();
	
	do
	{
		getKey();
		moveBall();
		
		if (TAIL_LEN > 0)
		{
			for(i=TAIL_LEN;i>0;i--)
			{
				tail_buf[i] = tail_buf[i-1];
			}
			tail_buf[0] = getPos(ballx,bally);
		}
		
		if (bally == SCREEN_H-1)
		{
			resetScreen();
			putChar('B',2,0,255);
			putChar('R',6,0,255);
			putChar('E',10,0,255);
			putChar('A',14,0,255);
			putChar('K',18,0,255);
			putChar('O',6,6,255);
			putChar('U',10,6,255);
			putChar('T',14,6,255);
			display();
			usleep(100000);
			display();
			sleep(1);
			initGame();
		}
		
		//victory?
		
		writeField();
		display();		
		usleep(100000);
	}while(display());
	endwin();
	return 0;
}
