//gcc -lnsl -lresolv -lcurses maze.c -o maze && ./maze 127.0.0.1 1024

#include "TCP_Client.c"
#include <time.h>
#include "curses.h"

#define ANGULAR 1
#define SHOWBUILD 0

char* field;
int pos;

int differentAdjacent(unsigned short* nums, int pos)
{
	//when angular, then do not delete nodes
	if((pos%2==0) && ((pos/25)%2==0) && ANGULAR)
		return 0;
		
	//check up and down
	if((nums[pos-25]) && (nums[pos+25]))
	{
		if(nums[pos-25]==nums[pos+25])
			return 0;
	}
	
	//check left and right
	if((nums[pos-1]) && (nums[pos+1]))
	{
		if(nums[pos-1]==nums[pos+1])
			return 0;
	}
	return 1;
}

void setToNeighbor(unsigned short* nums, int pos1, int pos2)
{
	//printf("wall: %i, pos1: %i(%i), pos2: %i(%i)\n",array[pos2],pos1,nums[pos1]
	//no wall in this direction?
	if(field[pos2]==0)
	{
		if(nums[pos2]<nums[pos1])
		{
			nums[pos1]=nums[pos2];
			decreaseAdjacent(nums,pos1);
		}
		if(nums[pos2]>nums[pos1])
			decreaseAdjacent(nums,pos2);
	}
}

//sets numbers to smallest value in path beginning from position x,y
//works recursivly
void decreaseAdjacent(unsigned short* nums, int pos)
{
	//printf("decrease %i value %i\n",pos, nums[pos]);
	//sleep(1);
	setToNeighbor(nums,pos,pos-25);
	setToNeighbor(nums,pos,pos+1);
	setToNeighbor(nums,pos,pos+25);
	setToNeighbor(nums,pos,pos-1);
}

void createMaze()
{
	int i;
	unsigned short* nums=calloc(sizeof(unsigned short)*275,1);
	for(i=0; i<275; i++)
	{
		if(((i+(i/25))%2==0) || ((i/25)%2==0))
		{
			//set wall
			field[i]=1;			
		}
		else
			field[i]=0;
		nums[i]=i;
	}
	
	nums[26]=1;
	pos=26;
	field[273]=0;
	while(nums[248]!=1)
	{
		int r=rand()%250;
		//printf("%i\n",r);
		//sleep(1);
		if((field[r]==1) && (r>25) && (r<250) && (r%25) && ((r+1)%25))
		{		
			//wall, not on edges
			if(differentAdjacent(nums,r))
			{
				//can be deleted
				field[r]=0;
				//printf("decreasing...\n");
				decreaseAdjacent(nums,r);
				if(SHOWBUILD)
				{
					writeArray();
					display();
					usleep(100000);
				}
				//printf("decreased. 248: %i\n", nums[248]);
				//sleep(1);
			}			
		}		
	}
	//printf("finished\n");
	//sleep(1);
	free(nums);
	field[26]=1;
}

void getKey()
{
	int ch=getch();
	if(((ch==KEY_UP) || (ch=='w')) && field[pos-25]==0)
	{
		field[pos-25]=field[pos];
		field[pos]=0;
		pos=pos-25;
	}
	if(((ch==KEY_RIGHT) || (ch=='d')) && field[pos+1]==0)
	{
		field[pos+1]=field[pos];
		field[pos]=0;
		pos=pos+1;
	}
	if(((ch==KEY_DOWN) || (ch=='s')) && field[pos+25]==0)
	{
		field[pos+25]=field[pos];
		field[pos]=0;
		pos=pos+25;
	}
	if(((ch==KEY_LEFT) || (ch=='a')) && field[pos-1]==0)
	{
		field[pos-1]=field[pos];
		field[pos]=0;
		pos=pos-1;
	}
	if(ch==KEY_F(5))
	{
		createMaze();
	}
	if(ch==27)
	{
		//free(field);
		endwin();
		exit(1);
	}
}

void writeArray()
{
	resetScreen();
	int i;
	for(i=0; i<275; i++)
	{
		if(field[i]>=1)
			writeToArray(255,i);
		/*if(field[i]>1)
		{
			if(field[i]<15)
				writeToArray(field[i],i);
			else
				writeToArray(30-field[i],i);
			
		}*/
			
	}
}

int main(int argc, char *argv[])
{
	if(argc!=3)
	{
		printf("Usage: ./maze <Servername> <Serverport>\n");
		exit(1);
	}
	srand(time(NULL));
	init(argv[1],atoi(argv[2]));
	initscr();
	noecho();
	cbreak();
	keypad(stdscr, TRUE);
	nodelay(stdscr, TRUE);
	
	field=calloc(275,1);

	createMaze();

	while((display() && (pos!=273)))
	{
		getKey();
		
		//field[pos]=((field[pos]-4)%21)+5; //between 5 and 25
		//field[pos]=1-field[pos]; //turning pos on and off
		field[pos]=(field[pos]+1)%5;
		/*field[pos]+=1;
		if(field[pos]==16)
			field[pos]=-14;*/
		writeArray();
		display();
		usleep(75000);
	}
	endwin();
	return 0;
}
