//gcc -lnsl -lresolv -lcurses tetris.c -o tetris && ./tetris 127.0.0.1 1024

#include "TCP_Client.c"
#include <time.h>
#include "curses.h"
#include <error.h>


#define SCREEN_W 25
#define SCREEN_H 11
#define MAX_CYCLES 20
#define MIN_CYCLES 7
#define ROTATE_KEYS 0

struct block {
	char x[4];
	char y[4];
	unsigned char color;} current;
	
int I_block[] = {
	0b0000111100000000,
	0b0100010001000100,
	0b0000111100000000,
	0b0100010001000100};
int T_block[] = {
	0b0000111001000000,
	0b0100110001000000,
	0b0100111000000000,
	0b0100011001000000};
int O_block[] = {
	0b0000011001100000,
	0b0000011001100000,
	0b0000011001100000,
	0b0000011001100000};
int L_block[] = {
	0b0100010001100000,
	0b0000011101000000,
	0b0000011000100010,
	0b0000001011100000};
int L2_block[] = {
	0b0010001001100000,
	0b0000010001110000,
	0b0000011001000100,
	0b0000111000100000};
int Z_block[] = {
	0b0000110001100000,
	0b0010011001000000,
	0b0000110001100000,
	0b0010011001000000};
int S_block[] = {
	0b0000011011000000,
	0b0100011000100000,
	0b0000011011000000,
	0b0100011000100000};
	
	
int* blockTable[] = {
	I_block, T_block, O_block, 	//192
	L_block, L2_block,		//128
	Z_block, S_block};		//64

unsigned char** block_array;
unsigned int score;
unsigned int dcycles;
int* blockVector;
int rotation;
int block_x;
int block_y;

void parseCurrent(int b_mask)
{
	int i = 0;
	int j = 0;
	while(b_mask)
	{
		if (b_mask & 1)
		{
			current.x[j] = i%4;
			current.y[j] = i/4;
			j++;
		}
		b_mask = b_mask >> 1;
		i ++;
	}
}

void newBlock()
{
	int r = rand()%7;
	blockVector = blockTable[r];
	rotation = rand() % 4;
	int b_mask = blockVector[rotation];
	parseCurrent(b_mask);
	block_x = SCREEN_W;
	block_y = rand() % (SCREEN_H - 4 + 1);
	current.color = 64 * (rand() % 3 + 1);
}

void initGame()
{
	resetScreen();
	score = 0;
	dcycles = MAX_CYCLES;
	
	int i,j;
	for (i=0;i<SCREEN_H;i++)
	{
		for (j=0;j<SCREEN_W;j++)
			block_array[i][j] = 0;
	}
	
	newBlock();
	
	while(getch()!=ERR){}
}

void restart()
{
	resetScreen();
	putChar('S',1,0,255);
	putChar('C',5,0,255);
	putChar('O',9,0,255);
	putChar('R',13,0,255);
	putChar('E',17,0,255);
	putChar(':',21,0,255);
	int i = 21;
	do
	{
		putChar((score%10)+48,i,6,255);
		i -= 4;
		score = score/10;
	}while(score>0);
	display();
	sleep(1);
	initGame();
}

//test if column(s) are full
int deleteColumns()
{
	int i,j,k;
	int score_counter = 0;
	for (i=0; i<SCREEN_W;i++) 		//i: column to check
	{
		int c = 0;			//c: count empty spaces
		for (j=0; j<SCREEN_H;j++)	//j: traverse rows
		{
			if (!block_array[j][i]) //empty: increase counter
				c++;
		}
		if (c == SCREEN_H)	//empty column
			break;		
		if (c == 0)		//full column -> delete and collapse
		{
			score_counter = (score_counter==0)?1:2*score_counter;
			for (j=0; j<SCREEN_H;j++) //j: traverse rows
			{
				block_array[j][i] = 0; //delete home row
				for (k=0;k<SCREEN_W-1;k++) //k: traverse following columns
				{
					block_array[j][k+i] = block_array[j][k+i+1];
				}
			} 
			i--;
		}		
	}
	score += score_counter;
	dcycles = (score == 0)?dcycles:dcycles - score;
	if (dcycles < MIN_CYCLES)
		dcycles = MIN_CYCLES;	
	return score_counter;
}

int stopBlock()
{
	int i;
	for (i=0;i<4;i++)
	{
		int x = block_x + current.x[i];
		int y = block_y + current.y[i];
		block_array[y][x] = current.color;
		if (x == SCREEN_W-1)
			return -1;
	}
	return deleteColumns();
}

int moveBlock()
{
	int i;
	for (i=0;i<4;i++)
	{
		int x = block_x + current.x[i];
		int y = block_y + current.y[i];
		if (x == 0)
		{
			stopBlock();
			newBlock();
			return 1;
		}
		if ((x <= SCREEN_W) && (block_array[y][x-1]))
		{
			int r = stopBlock();			
			newBlock();
			return r;
		}		
	}
	block_x = block_x - 1;
	return 0;
}

void moveUp()
{
	int i;
	for (i=0;i<4;i++)
	{
		int x = block_x + current.x[i];
		int y = block_y + current.y[i];		
		if (y==0)
			return;

		if (block_array[y-1][x])
			return;
	}
	
	block_y -= 1;
}

void moveDown()
{
	int i;
	for (i=0;i<4;i++)
	{
		int x = block_x + current.x[i];
		int y = block_y + current.y[i];		
		if (y==SCREEN_H-1)
			return;

		if (block_array[y+1][x])
			return;
	}
	block_y += 1;
}

void hurry()
{
	if(moveBlock()==-1)
		restart();
}

void rotate()
{
	rotation = (rotation + 1)%4;
	int b_mask = blockVector[rotation];
	parseCurrent(b_mask);
	
	int i;
	for(i=0;i<4;i++)
	{
		//int x = block_x + current.x[i];
		//int y = block_y + current.y[i];	
		
		while(block_y + current.y[i] < 0)
			block_y++;
		while(block_y + current.y[i] >= SCREEN_H)
			block_y--;
		while(block_x + current.x[i] < 0)
			block_x++;
			
	}
}

void writeField(void)
{
	resetScreen();
	int x,y;
	for (y=0;y<SCREEN_H;y++)
	{
		for (x=0;x<SCREEN_W;x++)
		{
			putPixel(x,y,block_array[y][x]);
		}
	}
	
	int i;
	for (i=0;i<4;i++)
	{
		unsigned char x_pos = block_x + current.x[i];
		unsigned char y_pos = block_y + current.y[i];
		putPixel(x_pos,y_pos,current.color);
		//printf("%i:%i -> %i\n",x_pos,y_pos,current.color);
	}
	//printf("New circle\n");
}

void getKey()
{
	int ch=getch();
	while(ch!=ERR)
	{
		switch (ch)
		{
			case KEY_UP:
			case 'w':
				if (ROTATE_KEYS)
					rotate();
				else
					moveUp();
				break;
			case KEY_LEFT:
			case 'a':
				if (ROTATE_KEYS)
					moveUp();
				else
					hurry();
				break;
			case KEY_DOWN:
			case 's':
				if (ROTATE_KEYS)
					hurry();
				else
					moveDown();
				break;
			case KEY_RIGHT:
			case 'd':
				if (ROTATE_KEYS)
					moveDown();
				else
					rotate();
				break;
			case 27:	//ESC
			{
				endwin();
				exit(1);
				break;
			}
			default:
				break;	
		}
		writeField();		
		display();
		ch=getch();
	}
}

int main(int argc, char *argv[])
{
	if(argc!=3)
	{
		printf("Usage: ./tetris <Servername> <Serverport>\n");
		exit(1);
	}
	srand(time(NULL));
	init(argv[1],atoi(argv[2]));
	initscr();
	noecho();
	cbreak();
	keypad(stdscr, TRUE);
	nodelay(stdscr, TRUE);
		
	
	block_array = malloc(SCREEN_H*sizeof(char*));
	int i;
	for (i=0;i<SCREEN_H;i++)
		block_array[i] = calloc(SCREEN_W,sizeof(unsigned char));
	
	initGame();
	i = 0;
	do
	{
		i++;
		getKey();
		if (i >= dcycles)
		{
			i = 0;
			//if (dcycles>50)
			//	dcycles = dcycles - 1;
				
			//printf("%i cycles to go\n",dcycles);
			
			if(moveBlock()==-1)
				restart();
			writeField();		
			display();			
		}		
		usleep(10000);
	}while(display());
	endwin();
	return 0;
}
