//gcc -lnsl -lresolv -lcurses tron2p.c -o tron && ./tron 127.0.0.1 1024 1025
//gcc -lnsl -lresolv -lcurses tron2p.c -o tron && ./tron 127.0.0.1 1025

#include "TCP_Client.c"
#include <time.h>
#include "curses.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
//#include <err.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

int left, right;
char leftdir, rightdir;
char* field;

char getKey()
{
	int key=0;
	key=getch();
	//printf("%i\n",key);
	switch(key)
	{
		case KEY_UP:
		case 'w':
			return -25;
			break;
		case KEY_RIGHT:
		case 'd':
			return 1;
			break;
		case KEY_DOWN:
		case 's':
			return 25;
			break;
		case KEY_LEFT:
		case 'a':
			return -1;
			break;
		case 27:
			endwin();
			exit(1);
		default:
			return 0;
	}
	return 0;
}

writeArray()
{
	resetScreen();
	int i;
	for(i=0;i<275;i++)
	{
		writeToArray(field[i],i);
	}
}

void newGame(int winner)
{
	resetScreen();
	if(winner==1)
	{
		putChar('L',5,0);
		putChar('E',9,0);
		putChar('F',13,0);
		putChar('T',17,0);
	}
	if(winner==2)
	{
		putChar('R',3,0);
		putChar('I',7,0);
		putChar('G',11,0);
		putChar('H',15,0);
		putChar('T',19,0);
	}
	if(winner)
	{
		putChar('W',3,6);
		putChar('I',7,6);
		putChar('N',11,6);
		putChar('S',15,6);
		putChar('!',19,6);
		display();
		sleep(1);
	}
	int i;
	for(i=3;i>=1;i--)
	{
		resetScreen();
		putNumber(i,10,1);
		display();
		sleep(1);
	}
	for(i=0; i<275;i++)
	{
		field[i]=0;
	}
	
	left=getPos(3,5);
	right=getPos(21,5);
	leftdir=1;
	rightdir=-1;
	field[left]=1;
	field[right]=1;
	//empty key-queue
	int k=getch();
	while(k!=ERR)
	{
		k=getch();
	}
}

int isWall(int pos,char dir)
{
	if((dir==-25) && (pos<25))
		return 1;
	if((dir==1) && ((pos+1)%25==0))
		return 1;
	if((dir==25) && (pos>249))
		return 1;
	if((dir==-1) && (pos%25==0))
		return 1;
	if(field[pos+dir]>0)
		return 1;
	return 0;
}

int movePlayer(int pos, char dir)
{
	if(isWall(pos,dir))
		return -1;
	field[pos+dir]=field[pos];
	return pos+dir;
}

int countAhead(int pos, char dir)
{
	int free=0;
	while(isWall(pos+(free*dir),dir)==0)
		free++;
	//printf("free ahead in %i: %i\n",dir,free);
	return free;
}

char turnLeft(char dir)
{
	switch(dir)
	{
		case -25:
			return -1;
		case -1:
			return 25;
		case 25:
			return 1;
		case 1:
			return -25;
	}
	return 0;
}

void planMove()
{
	//KI must be right!
	if(rand()%(countAhead(right,rightdir)+1)==0)
	{
		//printf("I'd really like to turn now\n");
		char leftTurn=turnLeft(rightdir);
		char rightTurn=-leftTurn;
		int leftleft=countAhead(right,leftTurn);
		int rightleft=countAhead(right,rightTurn);
		if(leftleft<rightleft)
		{
			if(rightleft)
				rightdir=rightTurn;
		}
		else
		{
			if(leftleft)
				rightdir=leftTurn;
		}
		//printf("new direction: %i\n",leftdir);
	}
}

void gameServer(int port)
{	
	field=malloc(275);
	
	int sockfd;	//tcp socket descriptor
	struct sockaddr_in addr;//this tcp adress
	socklen_t addr_len;	//length of tcp adress

	fd_set rfds;	//set for sockets at which data is expected
	struct timeval timeout;
	struct sockaddr_in c_addr; //connecting information
	int c_addr_len=sizeof(c_addr);
	int new_sockfd;
	int KI=1;		//KI on/off

	int numbytes;	//number of successfully read bytes
	char buffer[1];//(temporary) read-in buffer

	//create tcp socket
	sockfd=socket(PF_INET, SOCK_STREAM, IPPROTO_IP);
	if (sockfd == -1)
	{
		printf("could not create tcp-socket\n");
		exit(1);
	}
	//set up tcp adress
	addr.sin_family = AF_INET;
	addr.sin_port=htons(port);
	addr.sin_addr.s_addr=htonl(INADDR_ANY); //get messages from any adress
	memset(addr.sin_zero, '\0', sizeof(addr.sin_zero)); //reset sin_zero to 0
	addr_len=sizeof(addr);

	//bind socket to adress
	if (bind(sockfd, (struct sockaddr*) &addr, addr_len) == -1)
	{
		printf("could not bind tcp-socket\n");
		close(sockfd);
		exit(1);
	}

	//set tcp-port to listen
	if (listen(sockfd, 4) == -1)
	{
		printf("could not listen @ tcp-socket\n");
		close(sockfd);
		exit(1);
	}
	
	printf("connected to board and starting server %s:%i\n",inet_ntoa(addr.sin_addr),port);
	sleep(1);
	srand(time(NULL));	
	//key functions for input
	initscr();
	noecho();
	cbreak();
	keypad(stdscr, TRUE);
	nodelay(stdscr, TRUE);
	
	//infinite loop to receive messages
	newGame(0);
	while(1)
	{
		//reset read-set
		FD_ZERO(&rfds);
		//after selecting, socket has to be put back in set
		FD_SET(sockfd, &rfds);
		timeout.tv_sec=0;
		timeout.tv_usec=100000;
		//wait for action
		select(sockfd+1, &rfds, NULL, NULL, &timeout);
		if(FD_ISSET(sockfd, &rfds))
		{
			printf("message\n");
			if(KI)			
			{
				//new player at tcp-port
	
				//create socket for client
				new_sockfd=accept(sockfd, (struct sockaddr*) &(c_addr), (socklen_t*) &c_addr_len);
				printf("new player\n");
				KI=0;	
				newGame(0);
				//printf("now receiving\n");
			}
			//read from socket
			//un-comment this to receive messages
			/*while((numbytes=recv(new_sockfd, buffer, 1,0)) > 0)
			{
				//printf("%i bytes from %s per TCP\n", numbytes, inet_ntoa(c_addr.sin_addr));
		    		rightdir=*buffer;
		    		printf("received dir: %i\n",rightdir);
			}
			printf("player left\n");
				close(new_sockfd);
				KI=1;
				newGame(0);
			*/
			//un-comment this to have a game
			else if((numbytes=recv(new_sockfd, buffer, 1,0)) > 0)
			{
				//printf("%i bytes from %s per TCP\n", numbytes, inet_ntoa(c_addr.sin_addr));
		    		rightdir=*buffer;
		    		printf("received dir: %i\n",rightdir);
			}
			else
			{
				//oops, no message -> hung up
				printf("player left\n");
				close(new_sockfd);
				KI=1;
				newGame(0);
			}
			printf("\n");
		}
		//printf("KI: %i\n",KI);
		if(KI)
			planMove();
		char k=getKey();
		if((k)&&(k!=-leftdir))
			leftdir=k;
		//printf("leftdir: %i\n",leftdir);
		left=movePlayer(left,leftdir);
		//printf("left: %i\n",left);
		if(left==-1)
			newGame(2);
		else 
		{
			right=movePlayer(right,rightdir);
			//printf("right: %i\n",right);
			if(right==-1)
				newGame(1);
		}	
		writeArray();
		display();
	}
}

void gameClient(char* server_ip, int server_port)
{
	struct sockaddr_in their_addr; 	//connector's address information
	struct hostent *he;
	int numbytes;
	char buffer[1];
	*buffer=-1;

	//Resolve hostname to IP Address
	if ((he=gethostbyname(server_ip)) == NULL) {  // get the host info
		herror("gethostbyname");
		printf("cannot resolve host\n");
		exit(1);
	}

	//create socket
	sockfd = socket(PF_INET,SOCK_STREAM,IPPROTO_IP);
	if (sockfd==-1)
	{
		printf("could not create socket\n");
		exit(1);
	}

	//setup transport address
	their_addr.sin_family = AF_INET;     
	their_addr.sin_port = htons(server_port);
	their_addr.sin_addr = *((struct in_addr *)he->h_addr_list[0]);
	memset(their_addr.sin_zero, '\0', sizeof their_addr.sin_zero);

	//connecting to server
	if (connect(sockfd, (const struct sockaddr*) &their_addr, sizeof(their_addr))==-1)
	{
		printf("connecting failed\n");
		close(sockfd);
		exit(1);
	}
	printf("connected to host\n");
	sleep(1);
	send(sockfd, buffer, 1, 0);
	
	//key functions for input
	initscr();
	noecho();
	cbreak();
	keypad(stdscr, TRUE);
	//nodelay(stdscr, TRUE);
	
	char key=-1;
	do
	{
		char oldKey=key;
		key=getKey();
		if((key==0)||(key==-oldKey))
		{
			key=oldKey;
		}
		*buffer=key;
		/*if(rand()%2)
			key=turnLeft(key);
		if(rand()%2)
			key=-turnLeft(key);
		*buffer=key;
		printf("key: %i\n",key);*/
		//usleep(100000);
		//send(sockfd,buffer,1,0);
	}while(send(sockfd, buffer, 1, 0)!=-1);
	printf("finished\n");
	close(sockfd);
	endwin();
}

int main(int argc, char *argv[])
{
	if(argc==4)
	{
		//Server
		init(argv[1],atoi(argv[2]));
		gameServer(atoi(argv[3]));
		return 0;
	}
	if(argc==3)
	{
		//Client
		gameClient(argv[1],atoi(argv[2]));
		return 0;
	}
	printf("Usage: ./pong <Pi-IP> <Pi-Port> <own Port> or ./pong <Server-IP> <Server-Port>\n");
	return 1;
}
